data {
    int<lower = 3> K;
    array[K] int<lower = 0> Freq_Noise;
    array[K] int<lower = 0> Freq_Signal;
}
transformed data {
    vector[K] a;
    for (i in 1:K) {
        a[i] = 1.0;
    }
}
parameters {
    real pre_mu;
    real<lower = 0.0> pre_sigma;
    simplex[K] ic;
}
transformed parameters {
    real mu;
    real sigma;
    vector[K-1] prec;
    vector[K-1] c;
    vector[K] theta_n;
    vector[K] theta_s;

    mu = pre_mu * 2.0;
    sigma = pre_sigma * 2.0 + 0.01;

    prec[1] = ic[1];
    for (i in 2:K-1) {
        prec[i] = prec[i-1] + ic[i];
    }
    for (i in 1:K-1) {
        c[i] = logit(prec[i]);
    }

    theta_n[1] = normal_cdf(c[1] | 0.0, 1.0);  
    theta_n[K] = 1.0 - normal_cdf(c[K-1] | 0.0, 1.0); 
    for (k in 2:(K-1))
        theta_n[k] = normal_cdf(c[k] | 0.0, 1.0) -
                        normal_cdf(c[k-1] | 0.0, 1.0);   
    theta_s[1] = normal_cdf((c[1] - mu) / sigma | 0.0, 1.0);
    theta_s[K] = 1.0 - normal_cdf((c[K-1] - mu) / sigma | 0.0, 1.0);   
    for (k in 2:(K-1))
        theta_s[k] =normal_cdf((c[k] - mu) / sigma | 0.0, 1.0) -
                      normal_cdf((c[k-1] - mu) / sigma | 0.0, 1.0);}
model {
    ic ~ dirichlet(a);
    pre_mu ~ normal(0.0, 10.0);
    pre_sigma ~ exponential(0.1);
    Freq_Noise ~ multinomial(theta_n);
    Freq_Signal ~ multinomial(theta_s);
}
