import matplotlib.pyplot as plt
import numpy as np
import stan
import seaborn as sb
import scipy.stats as ss
import arviz as az
import pickle

fout = open('Results.txt', 'w')

K = int(input('K = '))
s = input('Data for Noise = ')
Data_Noise = []
for v in s.split():
    Data_Noise.append(int(v))
s = input('Data for Signal = ')
Data_Signal = []
for v in s.split():
    Data_Signal.append(int(v))

fout.write('\nData for Noise:\n')
fout.write('    {}\n'.format(Data_Noise))
fout.write('\nData for Signal:\n')
fout.write('    {}'.format(Data_Signal))

Data = {'K': K, 'Freq_Noise': Data_Noise, 'Freq_Signal': Data_Signal}

def f_init():
    return {'pre_sigma': 0.5}

with open('YN_rating.stan', 'r') as fstan:
    sm = stan.build(fstan.read(), data = Data)

fit = sm.sample(num_samples = 5000, num_chains=4, init=[f_init()]*4)
print(fit)

i_data = az.from_pystan(posterior = fit, posterior_model = sm)
smry = az.summary(i_data)  
print('Summary =\n',smry)
fout.write('\n\n')
fout.write('Summary...\n')
fout.write(smry.__str__())
fout.write('\n')

d_fm = fit.to_frame()


def CalcMAPEst(samples, a = 0.05, n_points = 2000):
    """
        Calculatte a MAP estimate from a KDE graph on [Lp, Up]
        Lp and Up are 100*a/2 and 100(1-a/2) percentile points of samples
    """
    Lp, Up = np.percentile(samples, [100 * a/2, 100 * (1 - a/2)])  # import numpy as np
    coord = np.linspace(Lp, Up, n_points) 
    est_pdf = ss.gaussian_kde(samples).pdf(coord)  # import scipy.stats as ss
    map_idx = np.argmax(est_pdf) 
    MAP_Est = coord[map_idx]       
    return MAP_Est, est_pdf[map_idx]


az.plot_trace(i_data, var_names = ['mu', 'sigma', 'c'])
plt.show()

mu = d_fm['mu']  
sigma = d_fm['sigma'] 
mu_mean = mu.mean()
mu_MAP = CalcMAPEst(mu)[0]
mu_q025, mu_med, mu_q975 = np.percentile(mu, [2.5, 50, 97.5]) 
sgm_mean = sigma.mean()
sgm_MAP = CalcMAPEst(sigma)[0]
sgm_q025, sgm_med, sgm_q975 = np.percentile(sigma, [2.5, 50, 97.5]) 
fout.write('\nmu:\n')
fout.write('    MAP est. = {0:.3f}\n'.format(mu_MAP))
fout.write('    med.     = {0:.3f}\n'.format(mu_med))
fout.write('    mean     = {0:.3f}\n'.format(mu_mean))
fout.write('    95% CI   = [{0:.3f}, {1:.3f}]\n'.format(mu_q025, mu_q975))

fout.write('\nsigma:\n')
fout.write('    MAP est. = {0:.3f}\n'.format(sgm_MAP))
fout.write('    med.     = {0:.3f}\n'.format(sgm_med))
fout.write('    mean     = {0:.3f}\n'.format(sgm_mean))
fout.write('    95% CI   = [{0:.3f}, {1:.3f}]\n'.format(sgm_q025, sgm_q975))

sb.kdeplot(mu)
plt.title((r'$\mu$') + (r'($\Delta$') + 'm) = ' +
           '{0:<.3f}(mode)'.format(mu_MAP), fontsize = 18)
plt.yticks([])
plt.ylabel('Density', fontsize = 14)
plt.xlabel('$\mu$', fontsize = 16)
plt.tight_layout()
plt.savefig('Figure_mu.png')
plt.show()

sb.kdeplot(sigma)
plt.title("$\sigma$ = {0:<.3f}(mode) ".format(sgm_MAP), fontsize = 18)
plt.yticks([])
plt.ylabel('Density', fontsize = 14)
plt.xlabel('$\sigma$', fontsize = 18)
plt.tight_layout()
plt.savefig('Figure_sgm.png')
plt.show()

c = []
for i in range(K-1):
    c.append(d_fm[f'c.{i+1}'])
c = np.array(c).T
             
z_values = np.arange(-4.0, 4.001, 0.1)
x_values = 1.0 - ss.norm(0.0, 1.0).cdf(z_values)
y_values = 1.0 - ss.norm(mu_MAP, sgm_MAP).cdf(z_values)

plt.figure(figsize = (5,5))
plt.plot(x_values, y_values, 'b-')
plt.title("ROC-curve", fontsize = 18)
plt.xlabel("P(False Alarm)", fontsize = 14)
plt.ylabel("P(Hit)", fontsize = 14)

cum_freq_n = []
cum_freq_s = []
for k in range(K):
    cum_freq_n.append(Data_Noise[K - 1 - k])
    cum_freq_s.append(Data_Signal[K - 1 - k])

for k in range(1, K):
    cum_freq_n[k] += cum_freq_n[k-1]
    cum_freq_s[k] += cum_freq_s[k-1]

x_fa = []
y_ht = []
for k in range(K-1):
    x_fa.append(cum_freq_n[k] / cum_freq_n[K-1])
    y_ht.append(cum_freq_s[k] / cum_freq_s[K-1])
plt.plot(x_fa, y_ht, 'ro', label = 'Data')

print('\nI am calculating...')

c_means = np.zeros(K-1)
c_meds = np.zeros(K-1)
c_MAPs = np.zeros(K-1)
c_q025s = np.zeros(K-1)
c_q975s = np.zeros(K-1)
for k in range(K-1):
    c_means[k] = c.T[k].mean()
    c_MAPs[k] = CalcMAPEst(c.T[k])[0]
    c_q025s[k], c_meds[k], c_q975s[k] = np.percentile(c.T[k], [2.5, 50, 97.5])

fout.write('\n')
for k in range(K-1):
    fout.write('\nC{}:\n'.format(k+1))
    fout.write('    MAP est. = {0:.3f}\n'.format(c_MAPs[k]))
    fout.write('    median   = {0:.3f}\n'.format(c_meds[k]))
    fout.write('    mean     = {0:.3f}\n'.format(c_means[k]))
    fout.write('    95% CI   = [{0:.3f}, {1:.3f}]\n'.format(
                    c_q025s[k], c_q975s[k]))
    
x_fa = []
y_ht = []

for k in range(K-1):
    x_fa.append(1.0 - ss.norm().cdf(c_MAPs[k]))  
    y_ht.append(1.0 - ss.norm(mu_MAP, sgm_MAP).cdf(c_MAPs[k]))  
plt.plot(x_fa, y_ht, 'gs', label = 'Prediction')

plt.legend(fontsize = 12)
plt.tight_layout()
plt.savefig('Figure_ROC.png')
plt.show()

for k in range(K-1):
    sb.kdeplot(c.T[k], label = '$C_{0}$: mode = {1:.3f}'.format(k+1, c_MAPs[k]))
title_str = "Criterions" 
plt.title(title_str, fontsize = 18)
plt.yticks([])
plt.ylabel('Density', fontsize = 14)
plt.xlabel('C', fontsize = 14)
plt.legend(fontsize = 12)
plt.tight_layout()
plt.savefig('Figure_Cs.png')
plt.show()

fout.close()

